<?php
namespace Plugin\DBMS;

use App\Config\Settings;
use Plugin\Exception\Error;

/**
 * Mysql handle for the scratchblog framework;
 * @author ibrahim
 *
 */
class MySQLHelper {
	
	/**
	 * Database connection object
	 * @var \mysqli $con
	 */
	private static $con;
	
	/**
	 * Environment data
	 * @var array mixed;
	 */
	private static $data;
	
	/**
	 * Connect to the database. Pass the con argument if you have a connection already in use.
	 * 
	 * @param \mysqli $con
	 * @return mysqli
	 */
	public static function connect($con = null){
		if (self::$con){
			return self::$con;
		}
		
		if ($con) {
			self::$con = $con;
			return $con;
		}
		
		self::$data = Settings::getDBInfo();
		self::$con = new \mysqli(self::$data["DB_HOST"], self::$data["DB_USER"], self::$data["DB_PASS"], self::$data["DB_NAME"]);
		if (self::$con->connect_errno){
			Error::throw500("Database Connection failed, check your credentials:\n\n".self::$con->connect_errno.": ".self::$con->connect_error );
		}
		self::$con->set_charset("utf8");
		return self::$con;
	}
	
	
	public static function close() {
		if (self::$con){
			self::$con->close();
		}
	}
	
	/**
	 * return the active connection
	 * @return \mysqli
	 */
	public static function getConnection(){
		return self::$con;
	}
	
	/**
	 * Execute queries like insert, update or delete
	 * @param \mysqli $con
	 * @param string $sql
	 * @param string $param
	 */
	public static function executeSQL($con,$sql,$params = null){
		$statement = $con->prepare($sql);
		
		if (!$statement) {
			Error::throw500("SQL Error: $sql\n\n{$con->error}");
		}
		
		$type = "";
		$arg = array();

		if ($params && is_array($params)){
			foreach($params as $param) {
				if (is_numeric($param)){
					$type .= "d";
				}else {
					$type .= "s";
				}
				$arg[] = $param;
			}
			call_user_method_array("bind_param", $statement, $arg);
		}
		$res = $statement->execute();
		
		if (!$res){
			Error::throw500("Looks like the query failed to execute.\n\nError: {$statement->error}\n\nQuery: {$sql}\n\nParameters:".implode(",", $arg));
		}
		
		// TODO: check different values for update vs insert
		return true;
		
	}
	
	/**
	 * Run a query and return the value
	 * @param \mysqli $con
	 * @param string $sql
	 * @param array $param
	 * @return array
	 */
	public static function query(\mysqli $con,$sql,$params = array()){
		
		$c = 0;
		if (is_array($params) && !empty($params)){
			$newSQL = preg_replace_callback("/(\?)/U", function() use ($con,&$c,$params){
				$val = "'".$con->real_escape_string($params[$c])."'";
				$c++;
				return $val;
			}, $sql);
			
			$result = $con->query($newSQL);
			if ($result){
				return $result;
			}else {
				Error::throw500("Query failed miserably: $newSQL\n\nError:".$con->error );
			}
		}
		
		$result = $con->query($sql);
		if (!$result){
			Error::throw500("Query failed miserably: $sql\n\nError:".$con->error );
		}
		
		return $result;
	}
	
}